package com.sksamuel.jmxc

import scala.collection.JavaConverters._
import javax.management.remote.{JMXConnector, JMXConnectorFactory, JMXServiceURL}

/** @author Stephen Samuel */
object JmxClientFactory {

    /**
     * Create a new client connected to the local process identified by the vm display name
     *
     * @param displayName the display name of the VM
     *
     * @return a connected client or None if no such display name was found
     */
    def fromVmDisplayName(displayName: String): Option[JmxClient] =
        JmxClientFactorySupport.localAttachables.find(_.name == displayName).map(arg => newClient(arg.address))

    /**
     * Attempt to create a client for any local process that has a domain that matches the given domain name.
     * This is very useful when you want to include jmcx in a deployable, and then find the local VM that contains the
     * beans for your project at runtime.
     *
     * @param name the name of the required domain
     *
     * @return optional jmx client for a local process that had a domain matching the param.
     */
    def fromDomainName(name: String): Option[JmxClient] = {
        localClients.filter(arg => {
            arg.domain(name) match {
                case None => false
                case Some(d) => true
            }
        }).headOption
    }

    def localClients = JmxClientFactorySupport.localAttachables.map(arg => newClient(arg.address))

    /**
     * Create a new client connected to the  address specified by the given jmx service url.
     *
     * @param connectorAddress
     *
     * @return a new remote attached JmxClient
     */
    def newClient(connectorAddress: String): JmxClient = newClient(connectorAddress, null, null)

    /**
     * Create a new client connected to the remote address specified by the given jmx service url
     * and validated with the provided credentials
     *
     * @param connectorAddress
     *
     * @param username the username
     *
     * @param password the password
     *
     * @return a new remote attached JmxClient
     */
    def newClient(connectorAddress: String, username: String, password: String): JmxClient = {
        val env = scala.collection.mutable.Map.empty[String, Array[String]]
        if (username != null)
            env.put(JMXConnector.CREDENTIALS, Array(username, password))
        _createClient(connectorAddress, env.toMap)
    }

    private def _createClient(connectorAddress: String, env: Map[String, Array[String]]) = {
        val serviceAddress = new JMXServiceURL(connectorAddress)
        val connector = JMXConnectorFactory.connect(serviceAddress, env.asJava)
        val conn = connector.getMBeanServerConnection
        new JmxClient(conn)
    }
}
