package com.sksamuel.jmxc

import scala.collection.JavaConverters._
import javax.management.{ObjectInstance, MBeanServerConnection, ObjectName}

/** @author Stephen Samuel */
class JmxDomain(val name: String, conn: MBeanServerConnection) {

    def objectNames: Array[ObjectName] = conn.queryNames(new ObjectName(name + ":*"), null).asScala.toArray
    def objectInstances: Array[ObjectInstance] = conn.queryMBeans(new ObjectName(name), null).asScala.toArray

    /**
     * Returns all MBeans for this JmxDomain at the time of invocation.
     * Each returned MBean is immutable with respect to the list of attributes. If the list of published MBeans on the domain
     * changes then this method will need to be re-invoked.
     *
     * @return all MBeans for this domain
     */
    def beans: Array[MBean] = _beans(name + ":*")

    /**
     * Returns all MBeans for this JmxDomain of the given type.
     *
     * @param type The type of MBeans to query for.
     *
     * @return all MBeans of the given type for this domain
     */
    def beans(`type`: String): Array[MBean] = _beans(name + ":type=" + `type` + ",*")

    /**
     * Returns the optional MBean for the given name
     *
     * @param name
     *
     * @return Some if the bean exists or None if not
     */
    def bean(name: String): Option[MBean] = _beans(name).headOption

    /**
     * Returns a set of the names of all the types of MBeans in this domain, in no particular order.
     *
     * @return
     */
    def types: Set[String] = objectNames.map(_.getKeyProperty("type")).toSet

    private def _beans(name: String): Array[MBean] =
        conn.queryMBeans(new ObjectName(name), null).asScala.map(new MBean(_, conn)).toArray

    override def toString = "[JmxDomain name=" + name + "]"
}
