package com.sksamuel.jmxc

import javax.management._

/**
 * @author Stephen Samuel
 *
 *         An immutable instance of a managed bean. All attribute names are fetched on creation and then cached.
 *         If the attribute list can change at runtime, then you would need to create a new instance of this class
 *         from the JmxDomain.
 *
 **/
class MBean(instance: ObjectInstance, conn: MBeanServerConnection) {

    val objectName    = instance.getObjectName
    val canonicalName = objectName.getCanonicalName

    private val info = conn.getMBeanInfo(objectName)

    val attributes: Array[MBeanAttribute] = info.getAttributes.map(arg => {
        MBeanAttribute(arg.getName, arg.getType, arg.getDescription, arg.isReadable, arg.isWritable, this)
    })
    def attribute(name: String): Option[MBeanAttribute] = attributes.find(_.name == name)
    private[jmxc] def setAttribute(bean: MBeanAttribute, value: Any) {
        conn.setAttribute(objectName, new Attribute(bean.name, value))
    }

    val operations: Array[MBeanOperation] = info.getOperations.map(arg => {
        MBeanOperation(arg.getName, arg.getReturnType, arg.getSignature, arg.getDescription, arg.getImpact)
    })

    def attributeNames: Array[String] = attributes.map(_.name)
    def attributeValue(name: String): Any = conn.getAttribute(objectName, name)
    def attributeValue(attr: MBeanAttribute): Any = attributeValue(attr.name)
    def attributeValues = conn.getAttributes(objectName, attributeNames)
    def attributeValues(names: Array[String]): Any = conn.getAttributes(objectName, names)
    def attributeValues(attrs: Array[MBeanAttribute]): Any = attributeValues(attrs.map(_.name))

    def invoke(operation: MBeanOperation, params: AnyRef*): AnyRef = conn.invoke(objectName, operation.name, params.toArray, null)

    /**
     * Adds a notification listener to receive events originating from this bean
     *
     * @param listener the listener to register
     */
    def addNotificationListener(listener: NotificationListener) {
        conn.addNotificationListener(objectName, listener, null, null)
    }

    /**
     * Removes the given notification listener from listening to events from this MBean
     *
     * @param listener the listener to remove
     */
    def removeNotificationListener(listener: NotificationListener) {
        conn.removeNotificationListener(objectName, listener)
    }

    override def toString = "[MBean name=" + objectName + "]"
}

case class MBeanAttribute(name: String, className: String, description: String, readable: Boolean, writable: Boolean,
                          private val mbean: MBean) {
    def value(): Any = mbean.attributeValue(this)
    def value_$eq(value: Any) {
        mbean.setAttribute(this, value)
    }
}
case class MBeanOperation(name: String, returnType: String, signature: Seq[MBeanParameterInfo], description: String, impact: Int)